local objFrame = CreateFrame("Frame");

objFrame:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED");
objFrame:RegisterEvent("ENCOUNTER_END");
objFrame:RegisterEvent("ENCOUNTER_START");
objFrame:RegisterEvent("LOOT_CLOSED");
objFrame:RegisterEvent("LOOT_OPENED");
objFrame:RegisterEvent("LOOT_SLOT_CLEARED");
objFrame:RegisterEvent("PARTY_LOOT_METHOD_CHANGED");
objFrame:RegisterEvent("PLAYER_ALIVE");
objFrame:RegisterEvent("PLAYER_REGEN_ENABLED");
objFrame:RegisterEvent("VARIABLES_LOADED");

-- Slash commands
-- ****************************** --
SLASH_BBDKP1 = "/bbdkp";

SlashCmdList["BBDKP"] = function(strMessage)
	local strCommand = string.lower(strMessage);

	if ( strCommand == "alter" ) then
		BBDKP_DialogAlter:Show();
	elseif ( BBDKP_Raid ~= nil and strCommand == "kill" ) then
		local _, _, _, _, _, _, _, intInstance = GetInstanceInfo();
		local intCurrent = time();

		objFrame:get_encounter(0, {
			["name"]			= BBDKP_DIALOG_DEFAULT_ENCOUNTER,
			["instance"]		= intInstance,
			["objective"]		= {[1] = 1},
			["datetime_from"]	= intCurrent,
			["datetime_till"]	= intCurrent,
		});
	elseif ( BBDKP_Raid ~= nil and strCommand == "wipe" ) then
		local _, _, _, _, _, _, _, intInstance = GetInstanceInfo();
		local intCurrent = time();

		objFrame:get_encounter(0, {
			["name"]			= BBDKP_DIALOG_DEFAULT_ENCOUNTER,
			["instance"]		= intInstance,
			["objective"]		= {[1] = 0},
			["datetime_from"]	= intCurrent,
			["datetime_till"]	= intCurrent,
		});
	elseif ( BBDKP_Raid == nil ) then
		BBDKP_DialogStart:Show();
	elseif ( BBDKP_Raid ~= nil ) then
		BBDKP_DialogStop:Show();
	end
end;
-- ****************************** --
-- Slash commands

-- Announcement functions
-- ****************************** --
objFrame.set_announcement = function(self, strType, intAmount, ...)
	if ( BBDKP.message[strType] ~= nil ) then
		local strMessage, arg1, arg2 = BBDKP.message[strType], ...;
		local strMessage = string.gsub(strMessage, "{AMOUNT}", intAmount);

		if ( strType == "ALTER" ) then
			strMessage = string.gsub(strMessage, "{RECEIVER}",	arg1);
			strMessage = string.gsub(strMessage, "{REASON}",	arg2);
		elseif ( strType == "BID_MAIN" ) then
			strMessage = string.gsub(strMessage, "{ITEM}",		arg1);
		elseif ( strType == "BID_OFF" ) then
			strMessage = string.gsub(strMessage, "{ITEM}",		arg1);
		elseif ( strType == "INTERVAL" ) then
			local arg1 = string.lower(SecondsToTime(arg1 - arg2, true, true, 3));

			local arg1 = arg1:gsub("(%d) |4(%a+):(%a+);", function(intUnit, strSingle, strPlural)
				if ( intUnit == 1 ) then return intUnit .." ".. strSingle; end
				if ( intUnit ~= 1 ) then return intUnit .." ".. strPlural; end
			end);

			strMessage = string.gsub(strMessage, "{DURATION}",	arg1);
		elseif ( strType == "KILL" ) then
			strMessage = string.gsub(strMessage, "{ENCOUNTER}",	arg1);
		elseif ( strType == "LOOT" ) then
			strMessage = string.gsub(strMessage, "{ITEM}",		arg2);
			strMessage = string.gsub(strMessage, "{RECEIVER}",	arg1);
		elseif ( strType == "START" ) then
			strMessage = string.gsub(strMessage, "{MAP}",		arg1);
		elseif ( strType == "STOP" ) then
			strMessage = string.gsub(strMessage, "{MAP}",		arg1);
		elseif ( strType == "WIPE" ) then
			strMessage = string.gsub(strMessage, "{ENCOUNTER}",	arg1);
		end

		for strChannel, objType in pairs(BBDKP.announce) do
			if ( objType[strType] == true ) then
				if ( string.len(strMessage) > 0 ) then
					SendChatMessage(strMessage, strChannel);
				end
			end
		end
	end
end;
-- ****************************** --
-- Announcement functions

-- Encounter functions
-- ****************************** --
objFrame.add_encounter = function(self, intObjective, strEncounter, intForce)
	if ( BBDKP_Database_Encounter[intObjective] ~= nil or intForce ~= nil ) then
		local intEncounter = intObjective;

		if ( BBDKP_Database_Encounter[intObjective] ~= nil ) then
			intEncounter = BBDKP_Database_Encounter[intObjective];
			strEncounter = GetInstanceLockTimeRemainingEncounter(intEncounter);
		end

		if ( BBDKP_Raid.encounter[intEncounter] == nil ) then
			local _, _, _, _, _, _, _, intInstance = GetInstanceInfo();
			local intCurrent = time();

			if ( BBDKP_Database_Encounter_Link[intObjective] ~= nil ) then
				BBDKP_Raid.encounter[intEncounter] = {
					["name"]			= strEncounter,
					["instance"]		= intInstance,
					["objective"]		= BBDKP_Database_Encounter_Link[intObjective],
					["datetime_from"]	= intCurrent,
					["datetime_till"]	= 0,
				};
			else
				BBDKP_Raid.encounter[intEncounter] = {
					["name"]			= strEncounter,
					["instance"]		= intInstance,
					["objective"]		= {[intObjective] = 0},
					["datetime_from"]	= intCurrent,
					["datetime_till"]	= 0,
				};
			end
		end
	end
end;

objFrame.get_encounter = function(self, intEncounter, objEncounter)
	local intStatus, objWindow = 1, _G["BBDKP_DialogEncounter_".. intEncounter];

	for _, intDead in pairs(objEncounter.objective) do
		intStatus = math.min(intStatus, intDead);
	end

	if ( objWindow == nil ) then
		objWindow = CreateFrame("Frame", "BBDKP_DialogEncounter_".. intEncounter, UIParent, "BBDKP_DialogEncounter");

		objWindow.submit:HookScript("OnClick", function(self)
			local objParent = self:GetParent();

			if ( objParent.data ~= nil ) then
				local intAmount, intCurrent = objWindow.amount:GetNumber(), time();
				local intStatus, strComment = objWindow.status:GetChecked(), objWindow.comment:GetText();
				local intEncounter = UIDropDownMenu_GetSelectedValue(objWindow.encounter);
				local objData, objRoster = objWindow.data, objFrame:get_roster(intAmount);
				local strEncounter = GetInstanceLockTimeRemainingEncounter(intEncounter);

				if ( objData.datetime_till == 0 ) then
					objData.datetime_till = time();
				end

				table.insert(BBDKP_Raid.log.encounter, {
					["comment"]			= strComment,
					["instance"]		= objData.instance,
					["encounter"]		= intEncounter,
					["status"]			= intStatus,
					["datetime_from"]	= objData.datetime_from,
					["datetime_till"]	= objData.datetime_till,
					["roster"]			= objRoster,
				});

				for strGUID, intAmount in pairs(objRoster) do
					objFrame:set_standing(strGUID, intAmount);
				end

				if ( intStatus == true ) then
					objFrame:set_announcement("KILL", intAmount, strEncounter);
				else
					objFrame:set_announcement("WIPE", intAmount, strEncounter);
				end

				objParent.data = nil;
			end
		end);
	end

	objWindow.data = {
		["name"]			= objEncounter.name,
		["instance"]		= objEncounter.instance,
		["encounter"]		= intEncounter,
		["datetime_from"]	= objEncounter.datetime_from,
		["datetime_till"]	= objEncounter.datetime_till,
		["status"]			= intStatus,
	};

	objWindow:Hide();
	objWindow:Show();
end;

objFrame.set_encounter = function(self, intObjective, intStatus, intForce)
	if ( BBDKP_Database_Encounter[intObjective] ~= nil or intForce ~= nil ) then
		local intEncounter = intObjective;

		if ( BBDKP_Database_Encounter[intObjective] ~= nil ) then
			intEncounter = BBDKP_Database_Encounter[intObjective];
		end

		if ( BBDKP_Raid.encounter[intEncounter] ~= nil ) then
			local objEncounter = BBDKP_Raid.encounter[intEncounter];
			local objObjective = objEncounter.objective;

			if ( objObjective[intObjective] ~= nil ) then
				local intCurrent = time();

				BBDKP_Raid.encounter[intEncounter]["objective"][intObjective] = intStatus;
				BBDKP_Raid.encounter[intEncounter]["datetime_till"] = intCurrent;
			end

			if ( BBDKP_Database_Encounter_Stop[intObjective] ~= nil ) then
				self:get_encounter(intEncounter, objEncounter);
			end
		end
	end
end;
-- ****************************** --
-- Encounter commands

-- Loot functions
-- ****************************** --
objFrame.get_loot = function(self)
	local intThreshold = GetLootThreshold();

	for intIndex = 1, GetNumLootItems() do
		local strTexture, strName, _, intQuality = GetLootSlotInfo(intIndex);

		if ( strTexture ~= nil and intQuality >= intThreshold ) then
			local objWindow = _G["BBDKP_DialogLoot_".. intIndex];

			if ( objWindow == nil ) then
				objWindow = CreateFrame("Frame", "BBDKP_DialogLoot_".. intIndex, UIParent, "BBDKP_DialogLoot");

				objWindow.submit:HookScript("OnClick", function(self)
					local objParent = self:GetParent();

					if ( objParent.data ~= nil ) then
						local intReceiver = UIDropDownMenu_GetSelectedValue(objParent.receiver);
						local intIndex = objParent.data.index;

						GiveMasterLoot(intIndex, intReceiver);
					end
				end);
			end

			objWindow.data = {
				["index"]	= intIndex,
				["name"]	= strName,
				["link"]	= GetLootSlotLink(intIndex),
				["quality"]	= intQuality,
				["texture"]	= strTexture,
			};

			objWindow:Show();
		end
	end
end;

objFrame.set_loot = function(self, intIndex)
	local objWindow = _G["BBDKP_DialogLoot_".. intIndex];

	if ( objWindow ~= nil and objWindow.data ~= nil ) then
		local intLoot, objData, intCurrent = objWindow.amount:GetNumber(), objWindow.data, time();
		local intReceiver = UIDropDownMenu_GetSelectedValue(objWindow.receiver);

		local strGUID = UnitGUID(GetMasterLootCandidate(objData.index, intReceiver));
		local _, _, _, _, _, strName, strRealm = GetPlayerInfoByGUID(strGUID);
		if ( strRealm == "" ) then strRealm = GetRealmName("player"); end
		local strCharacter, strItem = strName .."-".. strRealm, objData.link;

		table.insert(BBDKP_Raid.log.loot, {
			["guid"]		= strGUID,
			["item"]		= strItem,
			["amount"]		= intLoot,
			["datetime"]	= intCurrent,
		});

		self:set_standing(strGUID, 0 - intLoot);
		self:set_announcement("LOOT", intLoot, strCharacter, strItem);

		objWindow.data = nil;
	end
end;
-- ****************************** --
-- Loot functions

-- Roster functions
-- ****************************** --
objFrame.get_roster = function(self, intAmount)
	local arrReturn = {};

	for intIndex = 1, MAX_RAID_MEMBERS do
		local strCharacter = GetRaidRosterInfo(intIndex);

		if ( strCharacter ~= nil ) then
			local strGUID = UnitGUID(strCharacter);
			arrReturn[strGUID] = intAmount;
		end
	end

	return arrReturn;
end;

objFrame.set_roster = function(self, strGUID, intAmount)
	local intCredit, intDebit, objAmount = 0, 0, BBDKP_Raid.amount;

	if ( intAmount >= 0 ) then intCredit = intAmount;	end;
	if ( intAmount <= 0 ) then intDebit = intAmount;	end;

	BBDKP_Raid.amount["credit"] = objAmount.credit + intCredit;
	BBDKP_Raid.amount["debit"] = objAmount.debit + intDebit;
	BBDKP_Raid.amount["total"] = objAmount.total + intAmount;

	if ( BBDKP_Raid.roster[strGUID] == nil ) then
		local _, strClass, _, strRace, intGender, strName, strRealm = GetPlayerInfoByGUID(strGUID);
		if ( strRealm == "" ) then strRealm = GetRealmName("player"); end

		local strFaction = UnitFactionGroup(strName .."-".. strRealm);
		if ( strFaction == nil ) then strFaction = UnitFactionGroup(strName); end

		BBDKP_Raid.roster[strGUID] = {
			["amount"]	= {
				["credit"]	= intCredit,
				["debit"]	= intDebit,
				["total"]	= intAmount,
			},
			["name"]	= strName,
			["realm"]	= strRealm,
			["level"]	= UnitLevel(strGUID),
			["faction"]	= strFaction,
			["race"]	= strRace,
			["class"]	= strClass,
			["gender"]	= intGender,
		};
	else
		local objAmount = BBDKP_Raid.roster[strGUID]["amount"];

		BBDKP_Raid.roster[strGUID]["amount"]["credit"] = objAmount.credit + intCredit;
		BBDKP_Raid.roster[strGUID]["amount"]["debit"] = objAmount.debit + intDebit;
		BBDKP_Raid.roster[strGUID]["amount"]["total"] = objAmount.total + intAmount;
	end
end;
-- ****************************** --
-- Roster functions

-- Standing functions
-- ****************************** --
objFrame.get_standing_index = function(self, strName, strRealm, intLoop)
	if ( intLoop >= 10 ) then return nil; end

	for intIndex = 1, GetNumGuildMembers() do
		local strCharacter, _, _, _, _, _, _, strOfficer = GetGuildRosterInfo(intIndex);
		if ( strOfficer == "" ) then strOfficer = "0"; end

		if ( strCharacter == strName or strCharacter == strName .."-".. strRealm ) then
			local intStanding = tonumber(strOfficer);

			if ( tostring(intStanding) == strOfficer ) then
				return {["index"] = intIndex, ["standing"] = intStanding};
			else
				return self:get_standing_index(strOfficer, strRealm, intLoop + 1);
			end
		end
	end
end;

objFrame.get_standing = function(self, strGUID)
	local _, _, intRank = GetGuildInfo("player");

	if ( intRank ~= nil and BBDKP.setting.note == true ) then
		local intRank = GuildControlSetRank(intRank + 1);
		local _, _, _, _, _, _, _, _, _, _, intGet = GuildControlGetRankFlags();

		if ( intGet == true ) then
			local _, _, _, _, _, strName, strRealm = GetPlayerInfoByGUID(strGUID);
			if ( strRealm == "" ) then strRealm = GetRealmName("player"); end
			local objCharacter = self:get_standing_index(strName, strRealm, 0);

			if ( objCharacter ~= nil ) then
				return objCharacter.standing;
			end
		end
	end

	if ( BBDKP_Raid.roster[strGUID] ~= nil ) then
		return BBDKP_Raid.roster[strGUID]["amount"]["total"];
	end
end;

objFrame.set_standing = function(self, strGUID, intAmount)
	local _, _, intRank = GetGuildInfo("player");

	if ( intRank ~= nil and BBDKP.setting.note == true ) then
		local intRank = GuildControlSetRank(intRank + 1);
		local _, _, _, _, _, _, _, _, _, _, intGet, intSet = GuildControlGetRankFlags();

		if ( intGet == true and intSet == true ) then
			local _, _, _, _, _, strName, strRealm = GetPlayerInfoByGUID(strGUID);
			if ( strRealm == "" ) then strRealm = GetRealmName("player"); end
			local objCharacter = self:get_standing_index(strName, strRealm, 0);

			if ( objCharacter ~= nil ) then
				local intStanding = objCharacter.standing + intAmount;
--				GuildRosterSetOfficerNote(objCharacter.index, intStanding);
			end
		end
	end

	self:set_roster(strGUID, intAmount);
end;
-- ****************************** --
-- Standing functions

objFrame:SetScript("OnEvent", function(self, event, ...)
	if ( event == "COMBAT_LOG_EVENT_UNFILTERED" ) then
		if ( BBDKP_Raid ~= nil ) then
			local _, strEvent, _, strFromGUID, strFromName, _, _, strTillGUID, strTillName, _, _, intSpell = ...;
			local strFromType, _, _, _, _, intFrom = strsplit("-", strFromGUID);
			local strTillType, _, _, _, _, intTill = strsplit("-", strTillGUID);

			if ( strTillType == "Creature" and intTill ~= nil ) then
				local intCreature, strCreature = tonumber(intTill), strTillName;
				local strPrefix, strSuffix = strEvent:match("(%a+)_(.+)");

				if ( strPrefix == "UNIT" and strSuffix == "DIED" ) then
					self:set_encounter(intCreature, 1);
				elseif ( strPrefix == "SPELL" and strSuffix == "RESURRECT"  ) then
					self:set_encounter(intCreature, 0);
				elseif ( strSuffix == "DAMAGE" ) then
					self:add_encounter(intCreature, strCreature);
				end
			end
		end
	end

	if ( event == "ENCOUNTER_END" ) then
		print(event);
		print(...);

		if ( BBDKP_Raid ~= nil ) then
			local _, _, intTotal = GetInstanceLockTimeRemaining();
			local intEncounter, strEncounter, _, _, intStatus = ...;

			for intIndex = 1, intTotal do
				if ( strEncounter == GetInstanceLockTimeRemainingEncounter(intIndex) ) then
					self:set_encounter(intIndex, intStatus, true);
				end
			end
		end
	end

	if ( event == "ENCOUNTER_START" ) then
		print(event);
		print(...);

		if ( BBDKP_Raid ~= nil ) then
			local _, _, intTotal = GetInstanceLockTimeRemaining();
			local intEncounter, strEncounter, _, _ = ...;

			for intIndex = 1, intTotal do
				if ( strEncounter == GetInstanceLockTimeRemainingEncounter(intIndex) ) then
					self:add_encounter(intIndex, strEncounter, true);
				end
			end
		end
	end

	if ( event == "PARTY_LOOT_METHOD_CHANGED" ) then
		local intRaid = UnitInRaid("player");

		if ( intRaid ~= nil ) then
			local _, _, _, _, _, _, _, _, _, _, intMasterLoot = GetRaidRosterInfo(intRaid);

			if ( BBDKP_Raid == nil and intMasterLoot == true ) then
				BBDKP_DialogStart:Show();
			elseif ( BBDKP_Raid ~= nil and intMasterLoot == false ) then
				BBDKP_DialogStop:Show();
			end
		end
	end

	if ( event == "LOOT_CLOSED" ) then
		for intIndex = 1, MAX_RAID_MEMBERS do
			local objWindow = _G["BBDKP_DialogLoot_".. intIndex];

			if ( objWindow ~= nil ) then
				objWindow.data = nil;
				objWindow:Hide();
			end
		end
	end

	if ( event == "LOOT_OPENED" ) then
		local intRaid = UnitInRaid("player");

		if ( BBDKP_Raid ~= nil and intRaid ~= nil ) then
			local _, _, _, _, _, _, _, _, _, _, intMasterLoot = GetRaidRosterInfo(intRaid);
			if ( intMasterLoot == true ) then self:get_loot(); end
		end
	end

	if ( event == "LOOT_SLOT_CLEARED" ) then
		local intRaid = UnitInRaid("player");

		if ( BBDKP_Raid ~= nil and intRaid ~= nil ) then
			local _, _, _, _, _, _, _, _, _, _, intMasterLoot = GetRaidRosterInfo(intRaid);
			if ( intMasterLoot == true ) then self:set_loot(...); end
		end
	end

	if ( event == "PLAYER_ALIVE" or event == "PLAYER_REGEN_ENABLED" ) then
		if ( BBDKP_Raid ~= nil ) then
			local intCombat, intDead = UnitAffectingCombat("player"), UnitIsDead("player");
			local intProgress = IsEncounterInProgress();

			if ( intCombat == false and intDead == false and intProgress == false ) then
				for intEncounter, objEncounter in pairs(BBDKP_Raid.encounter) do
					self:get_encounter(intEncounter, objEncounter);
					BBDKP_Raid.encounter[intEncounter] = nil;
				end
			end
		end
	end

	if ( event == "VARIABLES_LOADED" ) then
		if ( not BBDKP ) then
			BBDKP = {};
		end

		if ( not BBDKP.amount ) then
			BBDKP.amount = {
				["ALTER"]		= 0,
				["INTERVAL"]	= 5,
				["KILL"]		= 25,
				["LOOT"]		= 50,
				["START"]		= 10,
				["STOP"]		= 10,
				["WIPE"]		= 10,
			};
		end

		if ( not BBDKP.announce ) then
			BBDKP.announce = {
				["GUILD"] = {
					["ALTER"]		= true,
					["BID_MAIN"]	= false,
					["BID_OFF"]		= false,
					["INTERVAL"]	= false,
					["KILL"]		= false,
					["LOOT"]		= false,
					["START"]		= false,
					["STOP"]		= false,
					["WIPE"]		= false,
				},
				["RAID"] = {
					["ALTER"]		= false,
					["BID_MAIN"]	= false,
					["BID_OFF"]		= false,
					["INTERVAL"]	= true,
					["KILL"]		= true,
					["LOOT"]		= true,
					["START"]		= true,
					["STOP"]		= true,
					["WIPE"]		= true,
				},
				["RAID_WARNING"] = {
					["ALTER"]		= false,
					["BID_MAIN"]	= true,
					["BID_OFF"]		= true,
					["INTERVAL"]	= false,
					["KILL"]		= false,
					["LOOT"]		= false,
					["START"]		= false,
					["STOP"]		= false,
					["WIPE"]		= false,
				},
			};
		end

		if ( not BBDKP.message ) then
			BBDKP.message = {
				["ALTER"]		= BBDKP_MESSAGE_ALTER,
				["BID_MAIN"]	= BBDKP_MESSAGE_BID_MAIN,
				["BID_OFF"]		= BBDKP_MESSAGE_BID_OFF,
				["INTERVAL"]	= BBDKP_MESSAGE_INTERVAL,
				["KILL"]		= BBDKP_MESSAGE_KILL,
				["LOOT"]		= BBDKP_MESSAGE_LOOT,
				["START"]		= BBDKP_MESSAGE_START,
				["STOP"]		= BBDKP_MESSAGE_STOP,
				["WIPE"]		= BBDKP_MESSAGE_WIPE,
			};
		end

		if ( not BBDKP.setting ) then
			BBDKP.setting = {
				["note"]		= true,
			};
		end

		if ( not BBDKP_Log ) then
			BBDKP_Log = {
				["alter"]	= {},
				["history"]	= {},
			};
		end
	end

	if ( event == "VARIABLES_LOADED" ) then
		BBDKP_Dialog_Hook(self);
		BBDKP_OptionPanel_Hook(self);
	end
end);

objFrame:SetScript("OnUpdate", function(self)
	if ( BBDKP_Raid ~= nil ) then
		local intInterval, intCurrent = BBDKP_Raid.interval, time();
		local intStart, intPrevious = BBDKP_Raid.datetime, BBDKP_Raid.datetime;

		for intDatetime, _ in pairs(BBDKP_Raid.log.interval) do
			intPrevious = math.max(intPrevious, intDatetime);
		end

		if ( intInterval ~= 0 and intCurrent >= intPrevious + 900 ) then
			local objRoster = objFrame:get_roster(intInterval);

			BBDKP_Raid.log.interval[intCurrent] = objRoster;

			for strGUID, intAmount in pairs(objRoster) do
				self:set_standing(strGUID, intAmount);
			end

			self:set_announcement("INTERVAL", intInterval, intCurrent, intStart);
		end
	end
end);
